@@ -13,7 +13,7 @@
 error_checks <- function(y, L1.x, L2.x, L2.unit, L2.reg, L2.x.scale, pcs,
                          folds, bin.proportion, bin.size, survey, census,
                          ebma.size, k.folds, cv.sampling, loss.unit, loss.fun,
-                         best.subset, lasso, pca, gb, svm, mrp, forward.select,
+                         best.subset, lasso, pca, gb, svm, mrp,
                          best.subset.L2.x, lasso.L2.x, gb.L2.x, svm.L2.x,
                          mrp.L2.x, gb.L2.unit, gb.L2.reg, lasso.lambda,
                          lasso.n.iter, uncertainty, boot.iter) {
@@ -170,7 +170,7 @@ error_checks <- function(y, L1.x, L2.x, L2.unit, L2.reg, L2.x.scale, pcs,
                  "' must contain integer numbers only.", sep = ""))
     }
 
-    if (!(folds_var == 1:max(folds_var))) {
+    if (!any((folds_var == 1:max(folds_var)))) {
       stop(paste("Fold variable '", folds,
                  "' must contain a sequence of integers running from 1 to ",
                  max(folds_var), ".", sep = ""))
@@ -642,7 +642,7 @@ error_checks <- function(y, L1.x, L2.x, L2.unit, L2.reg, L2.x.scale, pcs,
         }
 
         # Check if mrp.L2.x is in survey data
-        if (all(mrp.L2.x != "empty")){
+        if (all(mrp.L2.x != "empty") & all(mrp.L2.x != "")){
           if (!all(mrp.L2.x %in% colnames(survey))) {
             stop(cat(paste("Context-level variable '",
                            mrp.L2.x[which(!(mrp.L2.x %in% colnames(survey)))],
@@ -653,7 +653,7 @@ error_checks <- function(y, L1.x, L2.x, L2.unit, L2.reg, L2.x.scale, pcs,
         }
 
         # Check if mrp.L2.x is in census data
-        if (all(mrp.L2.x != "empty")){
+        if (all(mrp.L2.x != "empty") & all(mrp.L2.x != "")){
           if (!all(mrp.L2.x %in% colnames(census))) {
             stop(cat(paste("Context-level variable '",
                            mrp.L2.x[which(!(mrp.L2.x %in% colnames(census)))],
@@ -677,13 +677,6 @@ error_checks <- function(y, L1.x, L2.x, L2.unit, L2.reg, L2.x.scale, pcs,
                " must be either TRUE or FALSE.", sep = ""))
   }
 
-  # Check if forward.select is logical
-  if (!is.logical(forward.select)) {
-    stop(paste("The logical argument 'forward.select', indicating whether to",
-               " use forward selection instead of best subset selection,",
-               " must be either TRUE or FALSE.", sep = ""))
-  }
-
   # Check if boot.iter corresponds to uncertainty, i.e. NULL if uncertainty = FALSE
   if (!uncertainty){
     if(!is.null(boot.iter)) {
@@ -1013,36 +1006,57 @@ model_list_pca <- function(y, L1.x, L2.x, L2.unit, L2.reg = NULL) {
 #'   names of the loss-functions and the second is called value and contains the
 #'   loss-function scores.
 
-loss_function <- function(pred, data.valid,
-                          loss.unit = c("individuals", "L2 units"),
-                          loss.fun = c("MSE", "MAE", "cross-entropy"),
-                          y, L2.unit) {
+loss_function <- function(
+  pred, data.valid,
+  loss.unit = c("individuals", "L2 units"),
+  loss.fun = c("MSE", "MAE", "cross-entropy"),
+  y, L2.unit
+) {
+
+  # Determine type of dependent variable
+  # for continuous variables do not do cross-entropy and f1 score
+  if (
+    data.valid %>%
+      dplyr::pull(!!y) %>%
+      unique() %>%
+      length() > 2
+  ) {
+    # remove cross-entropy and f1 score from loss.fun
+    loss.fun <- loss.fun[!loss.fun %in% c("cross-entropy", "f1")]
+  }
 
   ## Loss functions
   # MSE
   mse <- mean_squared_error(
     pred = pred, data.valid = data.valid,
-    y = y, L2.unit = L2.unit)
+    y = y, L2.unit = L2.unit
+  )
 
   # MAE
   mae <- mean_absolute_error(
     pred = pred, data.valid = data.valid,
-    y = y, L2.unit = L2.unit)
+    y = y, L2.unit = L2.unit
+  )
 
   # binary cross-entropy
-  bce <- binary_cross_entropy(
-    pred = pred, data.valid = data.valid,
-    y = y, L2.unit = L2.unit)
+  bce <- suppressWarnings(
+    binary_cross_entropy(
+      pred = pred, data.valid = data.valid,
+      y = y, L2.unit = L2.unit
+    )
+  )
 
   # f1 score
   f1 <- f1_score(
     pred = pred, data.valid = data.valid,
-    L2.unit = L2.unit, y = y)
+    L2.unit = L2.unit, y = y
+  )
 
   # mean squared false error
   msfe <- mean_squared_false_error(
     pred = pred, data.valid = data.valid,
-    y = y, L2.unit = L2.unit)
+    y = y, L2.unit = L2.unit
+  )
 
   # Combine loss functions
   score <- mse %>%
@@ -1056,7 +1070,7 @@ loss_function <- function(pred, data.valid,
     dplyr::filter(measure %in% loss.fun) %>%
     dplyr::filter(level %in% loss.unit) %>%
     dplyr::group_by(measure) %>%
-    dplyr::summarise(value = mean(value), .groups = "drop" )
+    dplyr::summarise(value = mean(value), .groups = "drop")
 
   # Function output
   return(score)
@@ -1130,7 +1144,8 @@ mean_absolute_error <- function(pred, data.valid, y, L2.unit){
   out <- dplyr::tibble(
     measure = rep("MAE", 2),
     value = rep(NA, 2),
-    level = c( "individuals", "L2 units"))
+    level = c("individuals", "L2 units")
+  )
 
   # mae values
   values <- rep(NA, 2)
@@ -1171,33 +1186,37 @@ mean_absolute_error <- function(pred, data.valid, y, L2.unit){
 #'   variables: measure, value and level.
 
 
-binary_cross_entropy <- function(pred, data.valid,
-                                 loss.unit = c("individuals", "L2 units"),
-                                 y, L2.unit){
+binary_cross_entropy <- function(
+  pred, data.valid, loss.unit = c("individuals", "L2 units"), y, L2.unit
+) {
 
   # outcome
   out <- dplyr::tibble(
     measure = rep("cross-entropy", 2),
     value = rep(NA, 2),
-    level = c( "individuals", "L2 units")
+    level = c("individuals", "L2 units")
   )
 
   # cross-entropy values
   values <- rep(NA, 2)
 
   # loss unit = "individual"
-  values[1] <- (mean( data.valid[[y]] * log(pred) + (1 - data.valid[[y]]) * log(1 - pred)))*-1
+  values[1] <- (mean(
+    data.valid[[y]] * log(pred) + (1 - data.valid[[y]]) * log(1 - pred)
+  )) * -1
 
   # loss unit = "L2 units"
   l2 <- data.valid %>%
     dplyr::mutate(pred = pred) %>%
     dplyr::rowwise() %>%
-    dplyr::mutate(ce = .data[[y]] * log(pred) + (1 - .data[[y]]) * log(1 - pred) ) %>%
+    dplyr::mutate(
+      ce = .data[[y]] * log(pred) + (1 - .data[[y]]) * log(1 - pred)
+    ) %>%
     dplyr::ungroup() %>%
     dplyr::group_by(!! rlang::sym(L2.unit)) %>%
     dplyr::summarise(bce = mean(ce), .groups = "drop")
 
-  values[2] <- mean(dplyr::pull(.data = l2, var = bce)) *-1
+  values[2] <- mean(dplyr::pull(.data = l2, var = bce)) * -1
 
   out <- dplyr::mutate(out, value = values)
   return(out)
@@ -1219,14 +1238,14 @@ binary_cross_entropy <- function(pred, data.valid,
 
 
 
-f1_score <- function(pred, data.valid, y, L2.unit){
+f1_score <- function(pred, data.valid, y, L2.unit) {
 
   ## individual level
 
   # true positives
   tp_ind <- data.valid %>%
     dplyr::mutate(pval = ifelse(test = pred > 0.5, yes = 1, no = 0)) %>%
-    dplyr::select( !! rlang::sym(y), pval ) %>%
+    dplyr::select(!! rlang::sym(y), pval) %>%
     dplyr::filter(pval == 1 & !! rlang::sym(y) == 1) %>%
     dplyr::summarise(tp = sum(pval)) %>%
     dplyr::pull(var = tp)
@@ -1234,58 +1253,59 @@ f1_score <- function(pred, data.valid, y, L2.unit){
   # false positives
   fp_ind <- data.valid %>%
     dplyr::mutate(pval = ifelse(test = pred > 0.5, yes = 1, no = 0)) %>%
-    dplyr::select( !! rlang::sym(y), pval ) %>%
-    dplyr::filter(pval == 1 & !! rlang::sym(y) == 0 ) %>%
+    dplyr::select(!! rlang::sym(y), pval) %>%
+    dplyr::filter(pval == 1 & !! rlang::sym(y) == 0) %>%
     dplyr::summarise(fp = sum(pval)) %>%
     dplyr::pull(var = fp)
 
   # false negatives
   fn_ind <- data.valid %>%
     dplyr::mutate(pval = ifelse(test = pred > 0.5, yes = 1, no = 0)) %>%
-    dplyr::select( !! rlang::sym(y), pval ) %>%
-    dplyr::filter(pval == 0 & !! rlang::sym(y) == 1 ) %>%
+    dplyr::select(!! rlang::sym(y), pval) %>%
+    dplyr::filter(pval == 0 & !! rlang::sym(y) == 1) %>%
     dplyr::summarise(fn = sum(!! rlang::sym(y))) %>%
     dplyr::pull(var = fn)
 
   # f1 score
-  f1 <- tp_ind / (tp_ind + 0.5 * (fp_ind + fn_ind) )
+  f1 <- tp_ind / (tp_ind + 0.5 * (fp_ind + fn_ind))
 
   # state-level f1 score
   state_out <- data.valid %>%
     # predicted values
     dplyr::mutate(pval = ifelse(test = pred > 0.5, yes = 1, no = 0)) %>%
     # select L2.unit, y, and predicted values
-    dplyr::select( !! rlang::sym(L2.unit), !! rlang::sym(y), pval ) %>%
+    dplyr::select(!! rlang::sym(L2.unit), !! rlang::sym(y), pval) %>%
     # group by L2.unit
-    dplyr::group_by( !! rlang::sym(L2.unit) ) %>%
+    dplyr::group_by(!! rlang::sym(L2.unit)) %>%
     # nest data
     tidyr::nest() %>%
     # new column with state-level f1 values
     dplyr::mutate(
-      f1 = purrr::map(data, function(x){
+      f1 = purrr::map(data, function(x) {
         # true positives
         tp <- x %>%
-          dplyr::select( !! rlang::sym(y), pval ) %>%
+          dplyr::select(!! rlang::sym(y), pval) %>%
           dplyr::filter(pval == 1 & !! rlang::sym(y) == 1) %>%
           dplyr::summarise(tp = sum(pval)) %>%
           dplyr::pull(var = tp)
         # false positives
         fp <- x %>%
-          dplyr::select( !! rlang::sym(y), pval ) %>%
-          dplyr::filter(pval == 1 & !! rlang::sym(y) == 0 ) %>%
+          dplyr::select(!! rlang::sym(y), pval) %>%
+          dplyr::filter(pval == 1 & !! rlang::sym(y) == 0) %>%
           dplyr::summarise(fp = sum(pval)) %>%
           dplyr::pull(var = fp)
         # false negatives
         fn <- x %>%
-          dplyr::select( !! rlang::sym(y), pval ) %>%
-          dplyr::filter(pval == 0 & !! rlang::sym(y) == 1 ) %>%
+          dplyr::select(!! rlang::sym(y), pval) %>%
+          dplyr::filter(pval == 0 & !! rlang::sym(y) == 1) %>%
           dplyr::summarise(fn = sum(!! rlang::sym(y))) %>%
           dplyr::pull(var = fn)
         # f1 score
-        f1 <- tp / (tp + 0.5 * (fp + fn) ) })) %>%
+        f1 <- tp / (tp + 0.5 * (fp + fn)) 
+      })) %>%
     # unnest f1 values
     tidyr::unnest(f1) %>%
-    dplyr::select( !! rlang::sym(L2.unit), f1 ) %>%
+    dplyr::select(!! rlang::sym(L2.unit), f1) %>%
     dplyr::ungroup() %>%
     dplyr::summarise(f1 = mean(f1, na.rm = TRUE), .groups = "drop")
 
@@ -1293,7 +1313,8 @@ f1_score <- function(pred, data.valid, y, L2.unit){
   out <- dplyr::tibble(
     measure = c("f1", "f1"),
     value = c(1 - f1, 1 - dplyr::pull(.data = state_out, var = f1)),
-    level = c("individuals", "L2 units"))
+    level = c("individuals", "L2 units")
+  )
 
   return(out)
 
@@ -1315,32 +1336,32 @@ f1_score <- function(pred, data.valid, y, L2.unit){
 
 
 
-mean_squared_false_error <- function(pred, data.valid, y, L2.unit){
+mean_squared_false_error <- function(pred, data.valid, y, L2.unit) {
 
   ## individual level
   msfe_l1 <- data.valid %>%
     dplyr::mutate(pval = ifelse(test = pred > 0.5, yes = 1, no = 0)) %>%
-    dplyr::select( !! rlang::sym(y), pval ) %>%
-    dplyr::group_by( !! rlang::sym(y) ) %>%
-    dplyr::mutate(err = (!! rlang::sym(y) - pval) ) %>%
+    dplyr::select(!! rlang::sym(y), pval) %>%
+    dplyr::group_by(!! rlang::sym(y)) %>%
+    dplyr::mutate(err = (!! rlang::sym(y) - pval)) %>%
     dplyr::summarise(err_rates = mean(err), .groups = "drop") %>%
     dplyr::mutate(err_rates = err_rates^2) %>%
-    dplyr::summarise( msfe = sum(err_rates)) %>%
+    dplyr::summarise(msfe = sum(err_rates)) %>%
     dplyr::pull(var = msfe)
 
   ## group level
   msfe_l2 <- data.valid %>%
     dplyr::mutate(pval = ifelse(test = pred > 0.5, yes = 1, no = 0)) %>%
-    dplyr::select( !! rlang::sym(L2.unit), !! rlang::sym(y), pval ) %>%
-    dplyr::group_by( !! rlang::sym(L2.unit) ) %>%
+    dplyr::select(!! rlang::sym(L2.unit), !! rlang::sym(y), pval) %>%
+    dplyr::group_by(!! rlang::sym(L2.unit)) %>%
     tidyr::nest() %>%
-    dplyr::mutate(msfe = purrr::map(data, function(x){
+    dplyr::mutate(msfe = purrr::map(data, function(x) {
       msfe <- x %>%
-        dplyr::group_by( !! rlang::sym(y) ) %>%
-        dplyr::mutate(err = (!! rlang::sym(y) - pval) ) %>%
+        dplyr::group_by(!! rlang::sym(y)) %>%
+        dplyr::mutate(err = (!! rlang::sym(y) - pval)) %>%
         dplyr::summarise(err_rates = mean(err), .groups = "drop") %>%
         dplyr::mutate(err_rates = err_rates^2) %>%
-        dplyr::summarise( msfe = sum(err_rates)) %>%
+        dplyr::summarise(msfe = sum(err_rates)) %>%
         dplyr::pull(var = msfe)
     })) %>%
     tidyr::unnest(msfe) %>%
@@ -1352,7 +1373,8 @@ mean_squared_false_error <- function(pred, data.valid, y, L2.unit){
   out <- dplyr::tibble(
     measure = c("msfe", "msfe"),
     value = c(msfe_l1, msfe_l2),
-    level = c("individuals", "L2 units"))
+    level = c("individuals", "L2 units")
+  )
 
   return(out)
 
@@ -1374,12 +1396,12 @@ mean_squared_false_error <- function(pred, data.valid, y, L2.unit){
 #'   column that corresponds to the cross-validation rank of a parameter
 #'   combination across all loss function scores.
 
-loss_score_ranking <- function(score, loss.fun){
+loss_score_ranking <- function(score, loss.fun) {
 
   # tuning parameter names
   params <- names(score)[!names(score) %in% c("measure", "value")]
 
-  ranking <- lapply(loss.fun, function(x){
+  ranking <- lapply(loss.fun, function(x) {
     score %>%
       dplyr::filter(measure == x) %>%
       dplyr::arrange(value) %>%
@@ -1387,7 +1409,7 @@ loss_score_ranking <- function(score, loss.fun){
   })
 
   ranking <- dplyr::bind_rows(ranking) %>%
-    dplyr::group_by( !!!rlang::syms(params) ) %>%
+    dplyr::group_by(!!!rlang::syms(params)) %>%
     dplyr::summarise(rank = sum(rank), .groups = "drop") %>%
     dplyr::arrange(rank)
 
@@ -1429,9 +1451,9 @@ quiet <- function(x) {
 multicore <- function(cores = 1, type, cl = NULL) {
 
   # Start parallel processing
-  if (type == "open"){
+  if (type == "open") {
     # register clusters for windows
-    if( Sys.info()["sysname"] == "Windows" ){
+    if(Sys.info()["sysname"] == "Windows") {
       cl <- parallel::makeCluster(cores)
       doParallel::registerDoParallel(cl)
       parallel::clusterCall(cl, function(x) .libPaths(x), .libPaths())
@@ -1443,7 +1465,7 @@ multicore <- function(cores = 1, type, cl = NULL) {
   }
 
   # Stop parallel processing
-  if (type == "close"){
+  if (type == "close") {
     parallel::stopCluster(cl)
   }
 }
@@ -1465,40 +1487,51 @@ multicore <- function(cores = 1, type, cl = NULL) {
 predict_glmmLasso <- function(census, m, L1.x, lasso.L2.x, L2.unit, L2.reg) {
 
   # Fixed effects
-  fixed_effects <- as.matrix(cbind(1, as.data.frame(census)[, lasso.L2.x])) %*% cbind(m$coefficients)
+  fixed_effects <- as.matrix(
+    cbind(1, as.data.frame(census)[, lasso.L2.x])
+  ) %*% cbind(m$coefficients)
 
   # Individual-level random effects
   ind_ranef <- dplyr::select(.data = census, dplyr::one_of(L1.x))
-  ind_ranef[] <- base::Map(paste, names(ind_ranef), ind_ranef, sep = '')
-  ind_ranef <- cbind(apply(ind_ranef, 1, function(x){
+  ind_ranef[] <- base::Map(paste, names(ind_ranef), ind_ranef, sep = "")
+  ind_ranef <- cbind(apply(ind_ranef, 1, function(x) {
     sum(m$ranef[which(names(m$ranef) %in% x)])
   }))
 
   # State random effects
-  state_ranef <- cbind(paste(L2.unit, as.character(dplyr::pull(.data = census, var = L2.unit)), sep = ""))
-  state_ranef <- cbind(apply(state_ranef, 1, function(x){
-    if (x %in% names(m$ranef)){
+  state_ranef <- cbind(paste(
+    L2.unit, as.character(dplyr::pull(.data = census, var = L2.unit)), sep = ""
+  ))
+  state_ranef <- cbind(apply(state_ranef, 1, function(x) {
+    if (x %in% names(m$ranef)) {
       m$ranef[names(m$ranef) == x]
-    } else{
+    } else {
       0
-    }}))
+    }
+  }))
 
   # Region random effect
-  if(!is.null(L2.reg)){
-    region_ranef <- cbind(paste(L2.reg, as.character(as.data.frame(census)[, L2.reg]), sep = ""))
-    region_ranef <- cbind(apply(region_ranef, 1, function(x){
+  if (!is.null(L2.reg)) {
+    region_ranef <- cbind(paste(
+      L2.reg, as.character(as.data.frame(census)[, L2.reg]), sep = ""
+    ))
+    region_ranef <- cbind(apply(region_ranef, 1, function(x) {
       m$ranef[names(m$ranef) == x]
     }))
   }
 
   # Predictions
-  if(!is.null(L2.reg)){
+  if (!is.null(L2.reg)) {
     lasso_preds <- cbind(fixed_effects, ind_ranef, state_ranef, region_ranef)
-  } else{
+  } else {
     lasso_preds <- cbind(fixed_effects, ind_ranef, state_ranef)
   }
   lasso_preds <- base::apply(X = lasso_preds, MARGIN = 1, FUN = sum)
-  lasso_preds <- stats::pnorm(lasso_preds)
+
+  # binary DV through the link function
+  if (m$y %>% unique() %>% length() == 2) {
+    lasso_preds <- stats::pnorm(lasso_preds)
+  }
 
   return(lasso_preds)
 }
@@ -1526,45 +1559,65 @@ predict_glmmLasso <- function(census, m, L1.x, lasso.L2.x, L2.unit, L2.reg) {
 #' @export
 #' @export plot.autoMrP
 
-plot.autoMrP <- function(x, algorithm = "ebma", ci.lvl = 0.95, ...){
+plot.autoMrP <- function(x, algorithm = "ebma", ci.lvl = 0.95, ...) {
 
   # L2.unit identifier
   L2.unit <- names(x$classifiers)[1]
 
 
   # Error if requested algorithm was not fitted
-  if(! algorithm %in% names(x$classifiers) & algorithm != "ebma" ){
-    stop('The ', algorithm, ' classifier was not run. Re-run autoMrP() with the requested algorithm. Allowed choices are: "ebma", "best_subset", "lasso", "pca", "gb", "svm", and "mrp".')
+  if (! algorithm %in% names(x$classifiers) && algorithm != "ebma") {
+    stop(
+      'The ', algorithm, ' classifier was not run. Re-run autoMrP() with the ',
+      'requested algorithm. Allowed choices are: "ebma", "best_subset", "lasso",
+      "pca", "gb", "svm", and "mrp".'
+    )
   }
 
   # plot classifier if EBMA was not estimated
-  if( "EBMA step skipped (only 1 classifier run)" %in% x$ebma ) {
+  if ("EBMA step skipped (only 1 classifier run)" %in% x$ebma) {
     algorithm <- names(x$classifiers)[-1]
   }
 
   # plot data
-  if(algorithm == "ebma"){
+  if (algorithm == "ebma") {
     # EBMA summary
     plot_data <- x$ebma %>%
       dplyr::group_by(!! rlang::sym(L2.unit)) %>%
-      dplyr::summarise(median = stats::median(ebma, na.rm = TRUE),
-                       lb = stats::quantile(x = ebma, p = (1 - ci.lvl)*.5, na.rm = TRUE),
-                       ub = stats::quantile(x = ebma, p = ci.lvl + (1 - ci.lvl)*.5, na.rm = TRUE),
-                       .groups = "drop") %>%
+      dplyr::summarise(
+        median = stats::median(ebma, na.rm = TRUE),
+        lb = stats::quantile(x = ebma, p = (1 - ci.lvl) * .5, na.rm = TRUE),
+        ub = stats::quantile(
+          x = ebma, p = ci.lvl + (1 - ci.lvl)*.5, na.rm = TRUE
+        ),
+        .groups = "drop"
+      ) %>%
       dplyr::arrange(median) %>%
       dplyr::mutate(rank = dplyr::row_number()) %>%
       dplyr::mutate(rank = as.factor(rank)) %>%
-      dplyr::mutate(!!rlang::sym(L2.unit) := forcats::fct_reorder(!!rlang::sym(L2.unit), median))
-  } else{
+      dplyr::mutate(
+        !!rlang::sym(L2.unit) := forcats::fct_reorder(
+          !!rlang::sym(L2.unit), median
+        )
+      )
+  } else {
     # One of the classifiers
     plot_data <- x$classifiers %>%
       dplyr::group_by(!! rlang::sym(L2.unit)) %>%
       dplyr::select(all_of(L2.unit), contains(algorithm)) %>%
-      dplyr::summarise_all(.funs = list(median = ~ stats::quantile(x = ., probs = 0.5, na.rm = TRUE),
-                                        lb = ~ stats::quantile(x = ., probs = (1 - ci.lvl) *.5, na.rm = TRUE),
-                                        ub = ~ stats::quantile(x = ., probs = ci.lvl + (1 - ci.lvl) *.5, na.rm = TRUE))) %>%
+      dplyr::summarise_all(.funs = list(
+        median = ~ stats::quantile(x = ., probs = 0.5, na.rm = TRUE),
+        lb = ~ stats::quantile(x = ., probs = (1 - ci.lvl) * .5, na.rm = TRUE),
+        ub = ~ stats::quantile(
+          x = ., probs = ci.lvl + (1 - ci.lvl) * .5, na.rm = TRUE
+        )
+      )) %>%
       dplyr::arrange(median) %>%
-      dplyr::mutate(!!rlang::sym(L2.unit) := forcats::fct_reorder(!!rlang::sym(L2.unit), median))
+      dplyr::mutate(
+        !!rlang::sym(L2.unit) := forcats::fct_reorder(
+          !!rlang::sym(L2.unit), median
+        )
+      )
   }
 
   # y axis tick labels
@@ -1615,19 +1668,22 @@ plot.autoMrP <- function(x, algorithm = "ebma", ci.lvl = 0.95, ...){
 #' @export
 #' @export summary.autoMrP
 
-summary.autoMrP <- function(object, ci.lvl = 0.95, digits = 4, format = "simple",
-                            classifiers = NULL, n = 10, ...){
+summary.autoMrP <- function(
+  object, ci.lvl = 0.95, digits = 4, format = "simple", classifiers = NULL,
+  n = 10, ...) {
 
   # weights
-  if ( all(c("autoMrP", "weights") %in% class(object)) ){
+  if (all(c("autoMrP", "weights") %in% class(object))) {
 
-    # error message if weights summary called without running multiple classifiers
-    if (any(object == "EBMA step skipped (only 1 classifier run)")){
-      stop("Weights are not reported if the EBMA step was skipped. Re-run autoMrP with multiple classifiers.")
+    # error message if weights summary called without running
+    # multiple classifiers
+    if (any(object == "EBMA step skipped (only 1 classifier run)")) {
+      stop("Weights are not reported if the EBMA step was skipped. ",
+      "Re-run autoMrP with multiple classifiers.")
     }
 
     # weights vector to tibble
-    if( is.null(dim(object)) ){
+    if (is.null(dim(object))) {
       object <- dplyr::tibble(!!!object)
     }
 
@@ -1649,9 +1705,9 @@ summary.autoMrP <- function(object, ci.lvl = 0.95, digits = 4, format = "simple"
       dplyr::arrange(dplyr::desc(median))
 
     # weights with uncertainty
-    if ( all(s_data$median != s_data$min) ){
-      n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data) )
-      cat( paste("\n", "# EBMA classifier weights:"), sep = "")
+    if (all(s_data$median != s_data$min)) {
+      n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data))
+      cat(paste("\n", "# EBMA classifier weights:"), sep = "")
       # output table
       output_table(
         object = s_data[1:n, ],
@@ -1665,11 +1721,14 @@ summary.autoMrP <- function(object, ci.lvl = 0.95, digits = 4, format = "simple"
           "Max"),
         format = format,
         digits = digits)
-      if (n < nrow(s_data)) cat( paste("... with", nrow(s_data)-n, " more rows", "\n", "\n"), sep = "")
+      if (n < nrow(s_data)) {
+        cat(paste("... with", nrow(s_data) - n,
+          " more rows", "\n", "\n"), sep = "")
+      }
     } else{
       s_data <- dplyr::select(.data = s_data, method, median)
-      n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data) )
-      cat( paste("\n", "# EBMA classifier weights:"), sep = "")
+      n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data))
+      cat(paste("\n", "# EBMA classifier weights:"), sep = "")
       output_table(
         object = s_data[1:n, ],
         col.names = c(
@@ -1677,12 +1736,13 @@ summary.autoMrP <- function(object, ci.lvl = 0.95, digits = 4, format = "simple"
           "Weight"),
         format = format,
         digits = digits)
-      if (n < nrow(s_data)) cat( paste("... with", nrow(s_data)-n, " more rows", "\n", "\n"), sep = "")
+      if (n < nrow(s_data)) cat(
+        paste(
+          "... with", nrow(s_data)-n,
+          " more rows", "\n", "\n"), sep = "")
     }
-  }
-
-  # ensemble summary
-  else if ( all(c("autoMrP", "ensemble") %in% class(object)) ) {
+  } else if (all(c("autoMrP", "ensemble") %in% class(object))) {
+    # ensemble summary
 
     # unit identifier
     L2.unit <- names(object)[1]
@@ -1692,16 +1752,17 @@ summary.autoMrP <- function(object, ci.lvl = 0.95, digits = 4, format = "simple"
       dplyr::group_by(!! rlang::sym(L2.unit)) %>%
       dplyr::summarise(
         min = base::min(ebma, na.rm = TRUE),
-        lb = stats::quantile(x = ebma, probs = (1 - ci.lvl)*.5, na.rm = TRUE),
+        lb = stats::quantile(x = ebma, probs = (1 - ci.lvl) * .5, na.rm = TRUE),
         median = stats::quantile(x = ebma, probs = .5, na.rm = TRUE),
-        ub = stats::quantile(x = ebma, probs = ci.lvl + (1 - ci.lvl)*.5, na.rm = TRUE),
+        ub = stats::quantile(
+          x = ebma, probs = ci.lvl + (1 - ci.lvl) * .5, na.rm = TRUE),
         max = base::max(ebma, na.rm = TRUE),
         .groups = "drop"
       )
 
     # with or without uncertainty
-    if ( all(s_data$median != s_data$lb) ){
-      cat( paste("\n", "# EBMA estimates:"), sep = "")
+    if (all(s_data$median != s_data$lb)) {
+      cat(paste("\n", "# EBMA estimates:"), sep = "")
       # output table
       output_table(
         object = s_data[1:n, ],
@@ -1714,29 +1775,32 @@ summary.autoMrP <- function(object, ci.lvl = 0.95, digits = 4, format = "simple"
           "Max"),
         format = format,
         digits = digits)
-      if (n < nrow(s_data)) cat( paste("... with", nrow(s_data)-n, " more rows", "\n", "\n"), sep = "")
-
-    } else{
+      if (n < nrow(s_data)) {
+        cat(paste(
+          "... with", nrow(s_data)-n, " more rows", "\n", "\n"), sep = "")
+      }
+    } else {
       s_data <- dplyr::select(.data = s_data, dplyr::one_of(L2.unit), median)
-      n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data) )
-      cat( paste("\n", "# EBMA estimates:"), sep = "")
+      n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data))
+      cat(paste("\n", "# EBMA estimates:"), sep = "")
       output_table(
         object = s_data[1:n, ],
         col.names = c(L2.unit, "Estimate"),
         format = format,
         digits = digits)
-      if (n < nrow(s_data)) cat( paste("... with", nrow(s_data)-n, " more rows", "\n", "\n"), sep = "")
+      if (n < nrow(s_data)) {
+        cat(paste(
+          "... with", nrow(s_data)-n, " more rows", "\n", "\n"), sep = "")
+      }
     }
-  }
-
-  # classifier summary
-  else if ( all(c("autoMrP", "classifiers") %in% class(object)) ){
+  } else if (all(c("autoMrP", "classifiers") %in% class(object))){
+    # classifier summary
 
     # unit identifier
     L2.unit <- names(object)[1]
 
     # multiple classifiers
-    if (base::is.null(classifiers)){
+    if (base::is.null(classifiers)) {
 
       # point estimates for all classifiers
       s_data <- object %>%
@@ -1745,31 +1809,37 @@ summary.autoMrP <- function(object, ci.lvl = 0.95, digits = 4, format = "simple"
 
       # output table
       ests <- paste(names(object)[-1], collapse = ", ")
-      n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data) )
-      cat( paste("\n", "# estimates of classifiers: ", ests), sep = "")
-      output_table(object = s_data[1:n, ],
-                   col.names = names(s_data),
-                   format = format,
-                   digits = digits)
-      if (n < nrow(s_data)) cat( paste("... with", nrow(s_data)-n, " more rows", "\n", "\n"), sep = "")
-    } else{
+      n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data))
+      cat(paste("\n", "# estimates of classifiers: ", ests), sep = "")
+      output_table(
+        object = s_data[1:n, ],
+        col.names = names(s_data),
+        format = format,
+        digits = digits)
+      if (n < nrow(s_data)) {
+        cat(paste(
+          "... with", nrow(s_data)-n, " more rows", "\n", "\n"), sep = "")
+      }
+    } else {
 
       # summary statistics
       s_data <- object %>%
-        dplyr::select(dplyr::one_of(L2.unit,classifiers)) %>%
+        dplyr::select(dplyr::one_of(L2.unit, classifiers)) %>%
         dplyr::group_by(!! rlang::sym(L2.unit)) %>%
         dplyr::summarise_all(.funs = list(
           min = ~ base::min(x = ., na.rm = TRUE),
-          lb = ~ stats::quantile(x = ., probs = (1 - ci.lvl)*.5, na.rm = TRUE),
+          lb = ~ stats::quantile(
+            x = ., probs = (1 - ci.lvl) * .5, na.rm = TRUE),
           median = ~ stats::median(x = ., na.rm = TRUE),
-          ub = ~ stats::quantile(x = ., probs = ci.lvl + (1 - ci.lvl)*.5, na.rm = TRUE),
+          ub = ~ stats::quantile(
+            x = ., probs = ci.lvl + (1 - ci.lvl) * .5, na.rm = TRUE),
           max = ~ base::max(x = ., na.rm = TRUE)
           ))
 
       # with or without uncertainty
-      if( all(s_data$median != s_data$lb) ){
-        n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data) )
-        cat( paste("\n", "# estimates of", classifiers, "classifier"), sep = "")
+      if (all(s_data$median != s_data$lb)) {
+        n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data))
+        cat(paste("\n", "# estimates of", classifiers, "classifier"), sep = "")
         output_table(
           object = s_data[1:n, ],
           col.names = c(
@@ -1781,10 +1851,14 @@ summary.autoMrP <- function(object, ci.lvl = 0.95, digits = 4, format = "simple"
             "Max"),
           format = format,
           digits = digits)
-        if (n < nrow(s_data)) cat( paste("... with", nrow(s_data)-n, " more rows", "\n", "\n"), sep = "")
-      } else{
-        s_data <- dplyr::select(.data = s_data, dplyr::one_of(L2.unit), "median")
-        n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data) )
+        if (n < nrow(s_data)) {
+          cat(paste(
+            "... with", nrow(s_data) - n, " more rows", "\n", "\n"), sep = "")
+        }
+      } else {
+        s_data <- dplyr::select(
+          .data = s_data, dplyr::one_of(L2.unit), "median")
+        n <- ifelse(n <= nrow(s_data), yes = n, no = nrow(s_data))
         cat( paste("\n", "# estimates of", classifiers, "classifier"), sep = "")
         output_table(
           object = s_data[1:n, ],
@@ -1985,130 +2059,20 @@ log_spaced <- function(min, max, n){
 
 # Bootstrap function run inside the foreach loop
 
-boot_fun <- function(survey, L2.unit,
-                     y, L1.x, L2.x, mrp.L2.x, L2.reg, L2.x.scale,
-                     pcs, folds, bin.proportion, bin.size, census,
-                     k.folds, cv.sampling, loss.unit, loss.fun, best.subset,
-                     lasso, pca, gb, svm, mrp, forward.select, best.subset.L2.x,
-                     lasso.L2.x, pca.L2.x, pc.names, gb.L2.x, svm.L2.x,
-                     svm.L2.unit, svm.L2.reg, gb.L2.unit, gb.L2.reg,
-                     lasso.lambda, lasso.n.iter, gb.interaction.depth,
-                     gb.shrinkage, gb.n.trees.init, gb.n.trees.increase,
-                     gb.n.trees.max, gb.n.minobsinnode, svm.kernel,
-                     svm.gamma, svm.cost, ebma.tol, ebma.size, cores, verbose) {
+boot_fun <- function(
+  survey, L2.unit, y, L1.x, L2.x, mrp.L2.x, L2.reg, L2.x.scale,
+  pcs, folds, bin.proportion, bin.size, census, k.folds,
+  cv.sampling, loss.unit, loss.fun, best.subset, lasso, pca,
+  gb, svm, mrp, deep.mrp, best.subset.L2.x,
+  lasso.L2.x, pca.L2.x, pc.names, gb.L2.x, svm.L2.x, svm.L2.unit,
+  svm.L2.reg, gb.L2.unit, gb.L2.reg, deep.L2.x, deep.L2.reg,
+  deep.splines, lasso.lambda, lasso.n.iter, gb.interaction.depth,
+  gb.shrinkage, gb.n.trees.init, gb.n.trees.increase,
+  gb.n.trees.max, gb.n.minobsinnode, svm.kernel, svm.gamma,
+  svm.cost, ebma.tol, ebma.size) {
 
   # Bootstrap sample --------------------------------------------------------
 
-  # simple sampling with replacement
-  # boot_sample <- dplyr::slice_sample(
-  #   .data = survey, n = nrow(survey), replace = TRUE)
-
-  # number of states to draw in cluster balanced bootstrap
-  # avg_n <- survey %>% dplyr::mutate(nrows = dplyr::n()) %>%
-  #   dplyr::group_by(!!rlang::sym(L2.unit)) %>%
-  #   dplyr::mutate(state_n = dplyr::n(),
-  #                 state_proportion =  (dplyr::n() / nrows))%>%
-  #   dplyr::summarise(state_n = mean(state_n),
-  #                    state_proportion = mean(state_proportion),
-  #                    .groups = 'drop') %>%
-  #   dplyr::summarise(n = weighted.mean(x = state_n, w = state_proportion)) %>%
-  #   as.numeric
-  #
-  # #avg_n <- floor(x = (nrow(survey) / avg_n) )
-  # avg_n <- round(x = (nrow(survey) / avg_n), digits = 0)
-  #
-  # # balanced cluster bootstrap
-  # boot_sample <- survey %>%
-  #   dplyr::group_by(!!rlang::sym(L2.unit)) %>%
-  #   tidyr::nest() %>%
-  #   dplyr::left_join(
-  #     y = survey %>%
-  #       dplyr::mutate(nrows = dplyr::n()) %>%
-  #       dplyr::group_by(!!rlang::sym(L2.unit)) %>%
-  #       dplyr::mutate(state_proportion =  (dplyr::n() / nrows)) %>%
-  #       dplyr::summarise(state_proportion = mean(state_proportion),
-  #                        .groups = 'drop'), by = L2.unit) %>%
-  #   dplyr::ungroup() %>%
-  #   dplyr::slice_sample(n = avg_n, weight_by = state_proportion,
-  #                       replace = TRUE) %>%
-  #   dplyr::select(-state_proportion) %>%
-  #   tidyr::unnest(data)
-
-  # # drop observations if the bootstrap sample is too large
-  # if (nrow(boot_sample) > nrow(survey)){
-  #   boot_sample <- dplyr::slice_sample(.data = boot_sample, n = nrow(survey),
-  #                                      replace = TRUE)
-  # }
-
-  # # Sample 1) regions; 2) states; 3) individuals
-  # if (!is.null(L2.reg)) {
-  #   # Step 1: Sample regions but sample at least 2 different regions
-  #   boot_sample <- dplyr::bind_rows(
-  #       # Sample at least 2 different regions
-  #       dplyr::slice_sample(.data = survey %>%
-  #                             dplyr::group_by(!!rlang::sym(L2.reg)) %>%
-  #                             tidyr::nest() %>%
-  #                             dplyr::ungroup()
-  #                           , n = 2, replace = FALSE),
-  #       # Sample from all regions with replacement
-  #       dplyr::slice_sample(.data = survey %>%
-  #                             dplyr::group_by(!!rlang::sym(L2.reg)) %>%
-  #                             tidyr::nest() %>%
-  #                             dplyr::ungroup(),
-  #                           n = (length(unique(unlist(survey[, L2.reg]))) - 2),
-  #                           replace = TRUE)) %>%
-  #     tidyr::unnest(data) %>%
-  #     dplyr::ungroup() %>%
-  #   # Step 2: sample states with replacement
-  #     dplyr::group_by(!!rlang::sym(L2.unit)) %>%
-  #     tidyr::nest() %>%
-  #     dplyr::ungroup() %>%
-  #     dplyr::slice_sample(n = nrow(.), replace = TRUE) %>%
-  #     tidyr::unnest(data) %>%
-  #     dplyr::ungroup() %>%
-  #     # Step 3: Sample individuals with replacement
-  #     dplyr::group_by(!!rlang::sym(L2.unit)) %>%
-  #     tidyr::nest() %>%
-  #     dplyr::mutate(data = purrr::map(data, function(x){
-  #       data = dplyr::slice_sample(.data = x, n = nrow(x), replace = TRUE)})) %>%
-  #     tidyr::unnest(data) %>%
-  #     dplyr::ungroup()
-  # } else {
-  #   # Step 1: sample states with replacement
-  #   boot_sample <- survey %>%
-  #     dplyr::group_by(!!rlang::sym(L2.unit)) %>%
-  #     tidyr::nest() %>%
-  #     dplyr::ungroup() %>%
-  #     dplyr::slice_sample(n = nrow(.), replace = TRUE) %>%
-  #     tidyr::unnest(data) %>%
-  #     dplyr::ungroup() %>%
-  #     # Step 2: Sample individuals with replacement within states
-  #     dplyr::group_by(!!rlang::sym(L2.unit)) %>%
-  #     tidyr::nest() %>%
-  #     dplyr::mutate(data = purrr::map(data, function(x){
-  #       data = dplyr::slice_sample(.data = x, n = nrow(x), replace = TRUE)})) %>%
-  #     tidyr::unnest(data) %>%
-  #     dplyr::ungroup()
-  # }
-
-  # sample states with replacement & within states sample individuals with
-  # replacement
-  # Step 1: sample states with replacement
-  # boot_sample <- survey %>%
-  #   dplyr::group_by(!!rlang::sym(L2.unit)) %>%
-  #   tidyr::nest() %>%
-  #   dplyr::ungroup() %>%
-  #   dplyr::slice_sample(n = nrow(.), replace = TRUE) %>%
-  #   tidyr::unnest(data) %>%
-  #   dplyr::ungroup() %>%
-  #   # Step 2: Sample individuals with replacement within states
-  #   dplyr::group_by(!!rlang::sym(L2.unit)) %>%
-  #   tidyr::nest() %>%
-  #   dplyr::mutate(data = purrr::map(data, function(x){
-  #     data = dplyr::slice_sample(.data = x, n = nrow(x), replace = TRUE)})) %>%
-  #   tidyr::unnest(data) %>%
-  #   dplyr::ungroup()
-
   # cluster bootstrap - sample L2 units with replacement
   boot_sample <- survey %>%
     dplyr::group_by(!!rlang::sym(L2.unit)) %>%
@@ -2118,44 +2082,6 @@ boot_fun <- function(survey, L2.unit,
     tidyr::unnest(data) %>%
     dplyr::ungroup()
 
-  # state stratified sample
-  # boot_sample <- survey %>%
-  #   dplyr::group_by( !! rlang::sym(L2.unit) ) %>%
-  #   tidyr::nest() %>%
-  #   dplyr::mutate(data = purrr::map(data, function(x){
-  #     data = dplyr::slice_sample(.data = x, n = nrow(x), replace = TRUE)
-  #   })) %>%
-  #   tidyr::unnest(data) %>%
-  #   dplyr::ungroup()
-
-  # at least one observation per state and simple sample
-  # boot_sample <- survey %>%
-  #   dplyr::group_by(!! rlang::sym(L2.unit)) %>%
-  #   dplyr::mutate(state_pick = ifelse(
-  #     test = dplyr::row_number() == sample(x = 1:dplyr::n(), size = 1),
-  #     yes = 1, no = 0)) %>%
-  #   dplyr::ungroup() %>%
-  #   dplyr::group_by(state_pick) %>%
-  #   tidyr::nest() %>%
-  #   dplyr::mutate(data = ifelse(
-  #     test = state_pick == 0,
-  #     yes = purrr::map(data, function(x){
-  #       dplyr::slice_sample(.data = x, n = nrow(x), replace = TRUE)
-  #     }),
-  #     no = data
-  #   )) %>%
-  #   tidyr::unnest(data) %>%
-  #   dplyr::ungroup()
-
-  # no-bootstrapping (same data as original survey (for testing only))
-  # boot_sample <- survey
-
-
-  # do not predict outcomes for states that are not in boot_sample ----------
-  # boot_census <- census %>%
-  #   dplyr::filter( !!rlang::sym(L2.unit) %in% unique(dplyr::pull(
-  #     .data = boot_sample, var = !!rlang::sym(L2.unit))) )
-
   # Create folds ------------------------------------------------------------
 
   if (is.null(folds)) {
@@ -2163,13 +2089,14 @@ boot_fun <- function(survey, L2.unit,
     # EBMA hold-out fold
     ebma.size <- round(nrow(boot_sample) * ebma.size, digits = 0)
 
-    if(ebma.size>0){
-      ebma_folding_out <- ebma_folding(data = boot_sample,
-                                       L2.unit = L2.unit,
-                                       ebma.size = ebma.size)
+    if (ebma.size > 0) {
+      ebma_folding_out <- ebma_folding(
+        data = boot_sample,
+        L2.unit = L2.unit,
+        ebma.size = ebma.size)
       ebma_fold <- ebma_folding_out$ebma_fold
       cv_data <- ebma_folding_out$cv_data
-    } else{
+    } else {
       ebma_fold <- NULL
       cv_data <- boot_sample
     }
@@ -2182,7 +2109,7 @@ boot_fun <- function(survey, L2.unit,
       cv.sampling = cv.sampling)
   } else {
 
-    if (ebma.size > 0){
+    if (ebma.size > 0) {
       # EBMA hold-out fold
       ebma_fold <- boot_sample %>%
         dplyr::filter_at(dplyr::vars(dplyr::one_of(folds)),
@@ -2198,8 +2125,6 @@ boot_fun <- function(survey, L2.unit,
       dplyr::group_split(.data[[folds]])
   }
 
-
-
   # Run classifiers ---------------------------------------------------------
 
   # Estimate on 1 sample in autoMrP
@@ -2233,7 +2158,7 @@ boot_fun <- function(survey, L2.unit,
     gb = gb,
     svm = svm,
     mrp = mrp,
-    forward.select = forward.select,
+    deep.mrp = deep.mrp,
     best.subset.L2.x = best.subset.L2.x,
     lasso.L2.x = lasso.L2.x,
     pca.L2.x = pca.L2.x,
@@ -2244,6 +2169,9 @@ boot_fun <- function(survey, L2.unit,
     svm.L2.reg = svm.L2.reg,
     gb.L2.unit = gb.L2.unit,
     gb.L2.reg = gb.L2.reg,
+    deep.L2.x = deep.L2.x,
+    deep.L2.reg = deep.L2.reg,
+    deep.splines = deep.splines,
     lasso.lambda = lasso.lambda,
     lasso.n.iter = lasso.n.iter,
     gb.interaction.depth = gb.interaction.depth,
@@ -2257,4 +2185,5 @@ boot_fun <- function(survey, L2.unit,
     svm.cost = svm.cost,
     ebma.tol = ebma.tol
   )
+  return(boot_mrp)
 }
